<?php

    class xx_down  extends down_class {

        /**
         * 示例
         */
//         $base_path = dirname(__FILE__).'/data';
//        $download_config = array(
//        array('http://demo.xinxiuvip.com/Discuz_X3.4_SC_UTF8_20210119.zip', $base_path.'/discuz.zip'),
//        array('https://static.dismall.com/202111021444/c141f39fe8a693516afdf719f8599f7f/resource/da/e2121365165ae1ac88af0b9b64b49ac8.jpg?e039f2ce1d51150c&ILG', $base_path.'/p2.jpg'),
//        );
//        curl_down($download_config);

        /**
         * 初始化
         * @param  Array  $download_config   下载的文件设置
         * @param  Int    $max_process_num   最大开启的进程数量
         * @param  Int    $timeout           超时秒数
         * @param  String $logfile           日志文件路径
         * 返回下载数据个数
         */
        public static function curl_down($download_config,$max_process_num=2,$timeout =30,$logfile =''){
          $obj = new down_class($download_config, $max_process_num, $timeout,$logfile);
          $handle_num = $obj->download();
          return $handle_num;
        }

        /*
         * 设置下载路径
         * xx_down::down_file('down/cece/');
         */
        public static function down_file($set_file){
            $file = DISCUZ_ROOT.'data/xinxiuvip_network/down/';
            xx_cache::cache_file('down/log/index.htm');//格式化目录

            if ($set_file){
                $file = $file.$set_file;
            }

            if (!file_exists($file)){
                mkdir($file,0777,true);
            }
            return $file;
        }
    }


    /**
     * 多进程批量下载文件（使用php curl_multi_exec实现）
     * Date:    2017-07-16
     * Author:  fdipzone
     * Version: 1.0
     *
     * Func
     * public  download 下载处理
     * public  process  多进程下载
     * private to_log   将执行结果写入日志文件
     */
    class down_class {

        // 下载文件设置
        private $download_config = array();

        // 最大开启进程数量
        private $max_process_num = 10;

        // 超时秒数
        private $timeout = 10;

        // 日志文件
        private $logfile = null;

        /**
         * 初始化
         * @param  Array  $download_config   下载的文件设置
         * @param  Int    $max_process_num   最大开启的进程数量
         * @param  Int    $timeout           超时秒数
         * @param  String $logfile           日志文件路径
         */
        public function __construct($download_config, $max_process_num=10, $timeout=10, $logfile=''){
            $this->download_config = $download_config;
            $this->max_process_num = $max_process_num;
            $this->timeout = $timeout;

            //删除已有文件
            foreach($download_config as $k=>$config){
                if (file_exists($config[1])){
                    unlink($config[1]);
                }
            }

            // 日志文件
            if($logfile){
                $this->logfile = $logfile;
            }else{
                $str_logfile = xx_down::down_file('log/');//设置日志存放路径
                $this->logfile =$str_logfile.'down_'.date('Ymd').'.log';
            }
        }

        /**
         * 执行下载
         * @result Int
         */
        public  function download(){

            // 已处理的数量
            $handle_num = 0;

            // 未处理完成
            while(count($this->download_config)>0){

                // 需要处理的大于最大进程数
                if(count($this->download_config)>$this->max_process_num){
                    $process_num = $this->max_process_num;
                    // 需要处理的小于最大进程数
                }else{
                    $process_num = count($this->download_config);
                }

                // 抽取指定数量进行下载
                $tmp_download_config = array_splice($this->download_config, 0, $process_num);

                // 执行下载
                $result = $this->process($tmp_download_config);

                // 写入日志
                $this->to_log($tmp_download_config, $result);

                // 记录已处理的数量
                $handle_num += count($result);

            }

            return $handle_num;

        }

        /**
         * 多进程下载文件
         * @param  Array $download_config 本次下载的设置
         * @return Array
         */
        public  function process($download_config){

            // 文件资源
            $fp = array();

            // curl会话
            $ch = array();

            // 执行结果
            $result = array();

            // 创建curl handle
            $mh = curl_multi_init();

            // 循环设定数量
            foreach($download_config as $k=>$config){

                $ch[$k] = curl_init();
                $fp[$k] = fopen($config[1], 'a');
                curl_setopt($ch[$k], CURLOPT_URL, $config[0]);
                curl_setopt($ch[$k], CURLOPT_FILE, $fp[$k]);
                curl_setopt($ch[$k], CURLOPT_HEADER, 0);
                curl_setopt($ch[$k], CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch[$k], CURLOPT_USERAGENT, 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)');

                // 加入处理
                curl_multi_add_handle($mh, $ch[$k]);
            }

            $active = null;

            do{
                $mrc = curl_multi_exec($mh, $active);
            } while($active);

            // 获取数据
            foreach($fp as $k=>$v){
                fwrite($v, curl_multi_getcontent($ch[$k]));
            }

            // 关闭curl handle与文件资源
            foreach($download_config as $k=>$config){
                curl_multi_remove_handle($mh, $ch[$k]);
                fclose($fp[$k]);

                // 检查是否下载成功
                if(file_exists($config[1])){
                    $result[$k] = true;
                }else{
                    $result[$k] = false;
                }
            }

            curl_multi_close($mh);

            return $result;

        }

        /**
         * 写入日志
         * @param Array $data 下载文件数据
         * @param Array $flag 下载文件状态数据
         */
        private function to_log($data, $flag){

            // 临时日志数据
            $tmp_log = '';

            foreach($data as $k=>$v){
               $data_log =  array(
                    'time'=>time(),
                   'url'=>$v[0],
                   'file'=>$v[1],
                   'status'=>$flag[$k]
                );
                $data_str = serialize($data_log);
                $tmp_log .=  $data_str.PHP_EOL;
            }

            // 创建日志目录
            if(!is_dir(dirname($this->logfile))){
                mkdir(dirname($this->logfile), 0777, true);
            }

            // 写入日志文件
            file_put_contents($this->logfile, $tmp_log, FILE_APPEND);
        }

    }